/*
 * Scratchbox Remote Shell
 * Copyright (c) 2003 Timo Savola <timo.savola@movial.fi>
 *
 * sbrsh is licensed under GPL.
 * You can read the full license at http://www.gnu.org/licenses/gpl.txt
 */

#ifndef PROTOCOL_H
#define PROTOCOL_H

#include <stdint.h>
#include <pty.h>
#include "common.h"

#define PROTOCOL_VERSION     1

#define INTERNAL_ERROR_CODE  32767

/** Authentication information. */
typedef struct {
	char user[USER_SIZE];
	char pwd[PWD_SIZE];
} auth_info_t;

/** Information on the command execution. */
typedef struct {
	int16_t umount;
	char *target;
	char **opts;
	char **argv;
	char *curdir;
	char **env;
	int16_t is_tty;
	int16_t tty_row;
	int16_t tty_col;
	int16_t tty_xpixel;
	int16_t tty_ypixel;
} cmd_info_t;

extern unsigned int get_ptrv_len(void **vec);

extern int write_int16(int fd, int16_t i);
extern int write_int32(int fd, int32_t i);
extern int write_str(int fd, const char *str);
extern int write_strv(int fd, const char **strv);

extern int16_t read_int16(int fd);
extern int32_t read_int32(int fd);
extern char *read_str(int fd);
extern char **read_strv(int fd);

extern int write_buf(int fd, void *buf, int len);
extern int read_buf(int fd, void *buf, int len);

extern int write_auth(int fd, const char *user, const char *pwd);
extern int read_auth(int fd, auth_info_t *auth);

extern int write_cmd(int fd, int16_t umount, const char *target,
		     const char **opts, const char **argv, const char *curdir,
		     const char **env, int16_t is_tty, const struct winsize *);
extern int read_cmd(int fd, cmd_info_t *cmd);
extern void free_cmd(cmd_info_t *cmd);

/** Authentication reply.                      server -> client */
#define PTYPE_AUTH_OK   1

/** Request data from stdin.                   server -> client */
#define PTYPE_IN_REQ    2

/** Some data written into stdin.              client -> server */
#define PTYPE_IN_DATA   3

/** Request data from stdout.                  client -> server */
#define PTYPE_OUT_REQ   4

/** Some data written into stdout.             server -> client */
#define PTYPE_OUT_DATA  5

/** Request data from stderr.                  client -> server */
#define PTYPE_ERR_REQ   6

/** Some data written into stderr.             server -> client */
#define PTYPE_ERR_DATA  7

/** The return code of the command.            server -> client */
#define PTYPE_RC        8

/** An error message; the client should die.   server -> client */
#define PTYPE_ERROR     9

/** An informative message.                    server -> client */
#define PTYPE_MESSAGE  10

/** Protocol version.                          client <> server */
#define PTYPE_VERSION  11

/** The header of a packet sent by the server. The data follows... */
typedef struct {
	/** One of the PTYPE-values. */
	int16_t type;
	/** The amount of data. */
	int32_t size;
} phead_t;

extern int write_packet(int fd, int16_t type, void *buf, int32_t size);
extern int read_phead(int fd, phead_t *head);

extern int send_version(int fd);
extern int get_version(int fd);

#endif
